%%%% solver = use 0 for mosek, 1 for SDPT3, and 2 for sedumi 
%%%% Method = use 0 Direct Method and 1 for Indrect Method
%%%% Optimal_Control_Problem = use 0 finite-horizon deterministic ODC
%%%% and 1 for infinite-horizon deterministic ODC and 2 for stochastic ODC

function [Upper_Bound,Lower_Bound,K,W] = Main_ODC_Solver(A,B,C,X0,L,Q,R,p,Sigma_disturbance,Sigma_noise,options,alpha,beta)

n=size(A,1);
m=size(B,2);
r=size(C,1);

switch options(1)
    case 0,
       Optimal_Control_Problem=0;
    case 1,
        Optimal_Control_Problem=1;
    case 2,
        Optimal_Control_Problem=2;
    otherwise
        Optimal_Control_Problem=1;
end



Method = options(2);
Solver = options(3);
epsi = options(4);

if beta~=0,
    Method=1;
end

%%%%%------------------------------------------------------------------------------

if Optimal_Control_Problem == 0
    
   if epsi == 0 ,
        [W,K,Lower_Bound] = Reduced_SDP_Time_Domain(A,B,C,X0,p,L,Q,R,epsi,alpha,beta,Solver);
    
        if Method==0,
            [Upper_Bound] = Objective_Function_Time_Domain(A,B,C,X0,p,Q,R,K,alpha);
        else
         [K] = Indirect_Method_Time_Domain(A,B,C,X0,p,L,W,alpha,beta,Solver);
          [Upper_Bound] = Objective_Function_Time_Domain(A,B,C,X0,p,Q,R,K,alpha);
        end
   else
      [W,K,Lower_Bound] = Reduced_SDP_Time_Domain(A,B,C,X0,p,L,Q,R,0.0, alpha,beta,Solver); 
      [W,K,Lbnd] = Reduced_SDP_Time_Domain(A,B,C,X0,p,L,Q,R,epsi, alpha,beta,Solver);
      
      if Method==0,
            [Upper_Bound] = Objective_Function_Time_Domain(A,B,C,X0,p,Q,R,K,alpha);
      else
         [K] = Indirect_Method_Time_Domain(A,B,C,X0,p,L,W,alpha,beta,Solver);
          [Upper_Bound] = Objective_Function_Time_Domain(A,B,C,X0,p,Q,R,K,alpha);
       end
      
   end
   
elseif Optimal_Control_Problem ==2
    if epsi == 0
    
        [W,P,K,Lower_Bound] = Reduced_SDP_Stochastic(A,B,C,L,Q,R,Sigma_disturbance,Sigma_noise,alpha,beta,epsi,Solver);
    
        if Method==0,
            [Upper_Bound] = Objective_Function_Stochastic(A,B,C,Q,R,K,Sigma_disturbance,Sigma_noise,alpha);
        else 
            [K] = Indirect_Method_Lyap_domain(A,B,C,L,Q,R,P,Sigma_noise,alpha,beta,Solver);
            [Upper_Bound] = Objective_Function_Stochastic(A,B,C,Q,R,K,Sigma_disturbance,Sigma_noise,alpha);
        end
    else 
        [W_witherror,P_witherror,K_witherror,Lower_Bound] = Reduced_SDP_Stochastic(A,B,C,L,Q,R,Sigma_disturbance,Sigma_noise,alpha,beta,0,Solver);
        [W,P,K,Lower_Bound_witherror] = Reduced_SDP_Stochastic(A,B,C,L,Q,R,Sigma_disturbance,Sigma_noise,alpha,beta,epsi,Solver);
        
        if Method==0,
            [Upper_Bound] = Objective_Function_Stochastic(A,B,C,Q,R,K,Sigma_disturbance,Sigma_noise,alpha);
        else 
            [K] = Indirect_Method_Lyap_domain(A,B,C,L,Q,R,P,Sigma_noise,alpha,beta,Solver);
            [Upper_Bound] = Objective_Function_Stochastic(A,B,C,Q,R,K,Sigma_disturbance,Sigma_noise,alpha);
        end
    end
   
else
    
    if epsi == 0
    
        [W,P,K,Lower_Bound] = Reduced_SDP_Stochastic(A,B,C,L,Q,R,X0*X0',zeros(r,r),alpha,beta,epsi,Solver);
    
        if Method==0,
            [Upper_Bound] = Objective_Function_Stochastic(A,B,C,Q,R,K,X0*X0',zeros(r,r),alpha);
        else 
            [K] = Indirect_Method_Lyap_domain(A,B,C,L,Q,R,P,zeros(r,r),alpha,beta,Solver);
            [Upper_Bound] = Objective_Function_Stochastic(A,B,C,Q,R,K,X0*X0',zeros(r,r),alpha);
        end
    else 
        [W_witherror,P_witherror,K_witherror,Lower_Bound] = Reduced_SDP_Stochastic(A,B,C,L,Q,R,X0*X0',zeros(r,r),alpha,beta,0,Solver);
        [W,P,K,Lower_Bound_witherror] = Reduced_SDP_Stochastic(A,B,C,L,Q,R,X0*X0',zeros(r,r),alpha,beta,epsi,Solver);
        
        if Method==0,
            [Upper_Bound] = Objective_Function_Stochastic(A,B,C,Q,R,K,X0*X0',zeros(r,r),alpha);
        else 
            [K] = Indirect_Method_Lyap_domain(A,B,C,L,Q,R,P,zeros(r,r),alpha,beta,Solver);
            [Upper_Bound] = Objective_Function_Stochastic(A,B,C,Q,R,K,X0*X0',zeros(r,r),alpha);
        end
    end
    
end

%%%%%------------------------------------------------------------------------------

%%%% Lower_Bound = lower bound on optimal objective
%%%% Upper_Bound = upper bound on optimal objective
%%%% K = recovered near-optimal controller
if (Upper_Bound<Lower_Bound-1),
    Lower_Bound
    disp('Upper_Bound is inaccurate')
else
   Lower_Bound
   Upper_Bound
   Optimality_Degree=100-(((Upper_Bound-Lower_Bound)/Upper_Bound)*100)
end



if max(abs(eig(A+B*K*C)))<1
    disp('Closed-loop system is stable')
else
    disp('Closed-loop system is not stable')
end
    
disp('If you would like to see the controller, please enter K')
