function [W,P1,K,Lower_Bound] = Reduced_SDP_Stochastic(A,B,C,L,Q,R,Sigma_disturbance,Sigma_noise,alpha,beta,epsi,Solver)

cvx_begin 

if Solver==0,
    cvx_solver mosek
elseif Solver==1
    cvx_solver SDPT3
else
    cvx_solver sedumi
end


cvx_precision best

n= size(A,1);
m= size(B,2);
r= size(C,1);


Phi=linsolve(C,[eye(r),zeros(r,n-r)]);
Phi(1:n,r+1:n)=null(C);

mu1=0.99*(min(eig(Q*(Phi)*(Phi'))));
mu2=0.99*(min(eig(Sigma_noise)));

QT=real(sqrtm(Q-mu1*(inv(Phi))'*(inv(Phi))));
RT=real(sqrtm(R));
Sigma_noise_s=real(sqrtm(Sigma_noise-mu2*eye(r)));

variable W(n+n+m,n+n+m) symmetric
variable K(m,r)
variable P(n,n) symmetric
variable M(r,r) symmetric
expression G(n,n) %Denote inv(P) 
expression stabmatrix 
expression objmatrix
expression objmatrix2

G=Phi*W(1:n,n+1:n+n);

stabmatrix = [G-mu1*W(n+1:n+n,n+1:n+n) , G , (A*G+B*W(n+n+1:n+n+m,n+1:n+n))' , W(n+1:n+n,n+n+1:n+n+m);
            G , inv(Q-mu1*(inv(Phi))'*(inv(Phi))) , zeros(n,n), zeros(n,m) ;
            (A*G+B*W(n+n+1:n+n+m,n+1:n+n)) , zeros(n,n), G , zeros(n,m) ;
            W(n+n+1:n+n+m,n+1:n+n) , zeros(m,n) , zeros(m,n) , inv(R) ] ;
            
objmatrix = [P, eye(n); eye(n),G];
objmatrix2 = [M,(B*K)'; (B*K), G];

minimize (trace(P*Sigma_disturbance)+trace(M*Sigma_noise)+...
    mu2*trace(R*W(n+n+1:n+n+m,n+n+1:n+n+m))+vec(RT*K*Sigma_noise_s)'*vec(RT*K*Sigma_noise_s)+...
    alpha*trace(W(n+n+1:n+n+m,n+n+1:n+n+m))+epsi*trace(W))


subject to


for i=1:m,
    for j=1:r,
        if (L(i,j)==0)
            K(i,j)==0;
        end
    end
end

W(1:n,1:n) == eye(n);
W == semidefinite(n+n+m);
stabmatrix == semidefinite(3*n+m);
objmatrix == semidefinite(2*n);
objmatrix2 == semidefinite(n+r);
K*C*Phi==W(n+n+1:n+n+m,1:n);
G==G';

beta*trace(W(n+n+1:n+n+m,n+n+1:n+n+m))<=1;

Spar=L*sign(abs(C*Phi))*sign(abs((C*Phi)'))*L';

for i=1:m,
    for j=1:m
        if Spar(i,j)==0;
            W(n+n+i,n+n+j)==0;
        end
    end
end

cvx_end

P1=inv(G);
Lower_Bound=trace(P1*Sigma_disturbance)+alpha*trace(W(n+n+1:n+n+m,n+n+1:n+n+m))+...
    trace(M*Sigma_noise)+mu2*trace(R*W(n+n+1:n+n+m,n+n+1:n+n+m))+...
    vec(RT*K*Sigma_noise_s)'*vec(RT*K*Sigma_noise_s);

K=full(K);
cvx_clear



