function [W,K,Lower_Bound] = Reduced_SDP_Time_Domain(A,B,C,X0,p,L,Q,R,epsi,alpha,beta,solver)

cvx_begin 

if solver==0,
    cvx_solver mosek
elseif solver==1
    cvx_solver SDPT3
else
    cvx_solver sedumi
end


cvx_precision best

n= size(A,1);
m= size(B,2);
r= size(C,1);

Phi=linsolve(C,[eye(r),zeros(r,n-r)]);
Phi(1:n,r+1:n)=null(C);

mu=0.99*(min(eig(Q*(Phi)*(Phi'))));

QT=real(sqrtm(Q-mu*(inv(Phi))'*(inv(Phi))));
RT=real(sqrtm(R));



variable W(n+m+p+1,n+m+p+1) symmetric
variable K(m,r)

X=Phi*W(1:n,n+1:n+p+1);
U=W(n+p+1+1:n+p+1+m,n+1:n+p+1);


minimize  (vec(QT*X)'*vec(QT*X)+vec(RT*U)'*vec(RT*U)+epsi*trace(W)+...
    alpha*trace(W(n+p+1+1:n+p+1+m,n+p+1+1:n+p+1+m))+(mu*trace(W(n+1:n+p+1,n+1:n+p+1))))

subject to

X(:,1)==X0; 

for i=1:m,
    for j=1:r,
        if (L(i,j)==0)
            K(i,j)==0;
        end
    end
end

W(1:n,1:n) == eye(n);
W == semidefinite(n+m+p+1);
K*C*Phi==W(n+p+1+1:n+p+1+m,1:n);

beta*trace(W(n+p+1+1:n+p+1+m,n+p+1+1:n+p+1+m))<=1;

for i=1:p,
    X(:,i+1)==A* X(:,i)+B*U(:,i);
end


Spar=L*sign(abs(C*Phi))*sign(abs(Phi'*C'))*L';

for i=1:m,
    for j=1:m
        if Spar(i,j)==0;
            W(n+p+1+i,n+p+1+j)==0;
        end
    end
end

cvx_end


Lower_Bound=mu*trace(W(n+1:n+p+1,n+1:n+p+1))+ vec(QT*X)'*vec(QT*X)+ ...
    vec(RT*U)'*vec(RT*U)+alpha*trace(W(n+p+1+1:n+p+1+m,n+p+1+1:n+p+1+m));
K=full(K);

cvx_clear